@@ -180,17 +180,116 @@ int plug_on_recv_command(MDI_Comm comm) {
 
 
 
-/*! \brief Launch an MDI plugin
+/*! \brief Load the initialization function for a plugin
  *
  */
-int library_launch_plugin(const char* plugin_name, const char* options, void* mpi_comm_ptr,
-                          MDI_Driver_node_callback_t driver_node_callback,
-                          void* driver_callback_object) {
+int library_load_init(const char* plugin_name, void* mpi_comm_ptr,
+                      library_data* libd, int mode) {
   int ret;
   int driver_code_id = current_code;
   code* this_code = get_code(driver_code_id);
   MPI_Comm mpi_comm = *(MPI_Comm*) mpi_comm_ptr;
 
+  //
+  // Get the path to the plugin
+  // Note: Eventually, should probably replace this code with libltdl
+  //
+  char* plugin_path = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+
+  // Get the name of the plugin's init function
+  char* plugin_init_name = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+  if ( mode == 0 ) { // Load MDI_Plugin_init
+    snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_init_%s", plugin_name);
+  }
+  else { // Load MDI_Plugin_open
+    snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_open_%s", plugin_name);
+  }
+
+  // Attempt to load a python script
+  snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/%s.py", this_code->plugin_path, plugin_name);
+  if ( file_exists(plugin_path) ) {
+    libd->is_python = 1;
+    ret = python_plugin_init( plugin_name, plugin_path, mpi_comm_ptr );
+    if ( ret != 0 ) {
+      mdi_error("Error in python_plugin_init");
+      return -1;
+    }
+  }
+  else {
+    libd->is_python = 0;
+
+#ifdef _WIN32
+  // Attempt to open a library with a .dll extension
+  snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/lib%s.dll", this_code->plugin_path, plugin_name);
+  libd->plugin_handle = LoadLibrary( plugin_path );
+  if ( ! libd->plugin_handle ) {
+    // Unable to find the plugin library
+    free( plugin_path );
+    mdi_error("Unable to open MDI plugin");
+    return -1;
+  }
+
+  // Load a plugin's initialization function
+  libd->plugin_init = (MDI_Plugin_init_t) (intptr_t) GetProcAddress( libd->plugin_handle, plugin_init_name );
+  if ( ! libd->plugin_init ) {
+    free( plugin_path );
+    free( plugin_init_name );
+    FreeLibrary( libd->plugin_handle );
+    mdi_error("Unable to load MDI plugin init function");
+    return -1;
+  }
+
+#else
+  // Attempt to open a library with a .so extension
+  snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/lib%s.so", this_code->plugin_path, plugin_name);
+  libd->plugin_handle = dlopen(plugin_path, RTLD_NOW);
+  if ( ! libd->plugin_handle ) {
+
+    // Attempt to open a library with a .dylib extension
+    snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/lib%s.dylib", this_code->plugin_path, plugin_name);
+    libd->plugin_handle = dlopen(plugin_path, RTLD_NOW);
+    if ( ! libd->plugin_handle ) {
+      // Unable to find the plugin library
+      free( plugin_path );
+      free( plugin_init_name );
+      mdi_error("Unable to open MDI plugin");
+      return -1;
+    }
+  }
+
+  // Load a plugin's initialization function
+  libd->plugin_init = (MDI_Plugin_init_t) (intptr_t) dlsym(libd->plugin_handle, plugin_init_name);
+  if ( ! libd->plugin_init ) {
+    free( plugin_path );
+    free( plugin_init_name );
+    dlclose( libd->plugin_handle );
+    mdi_error("Unable to load MDI plugin init function");
+    return -1;
+  }
+#endif
+
+    // Initialize an instance of the plugin
+    ret = libd->plugin_init();
+    if ( ret != 0 ) {
+      mdi_error("MDI plugin init function returned non-zero exit code");
+      return -1;
+    }
+  }
+
+  // free memory from loading the plugin's initialization function
+  free( plugin_path );
+  free( plugin_init_name );
+
+  return 0;
+}
+
+
+
+/*! \brief Parse command-line plugin options
+ *
+ */
+int library_parse_options(const char* options, library_data* libd) {
+
   // Begin parsing the options char array into an argv-style array of char arrays
 
   // copy the input options array
@@ -201,7 +300,7 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   snprintf(plugin_unedited_options, options_len, "%s", options);
 
   // determine the number of arguments
-  plugin_argc = 0;
+  libd->plugin_argc = 0;
   int ichar;
   int in_argument = 0; // was the previous character part of an argument, or just whitespace?
   int in_single_quotes = 0; // was the previous character part of a single quote?
@@ -209,25 +308,25 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   for (ichar=0; ichar < options_len; ichar++) {
     if ( plugin_options[ichar] == '\0' ) {
       if ( in_double_quotes ) {
-	mdi_error("Unterminated double quotes received in MDI_Launch_plugin \"options\" argument.");
+        mdi_error("Unterminated double quotes received in MDI_Launch_plugin \"options\" argument.");
       }
       if ( in_argument ) {
-	plugin_argc++;
+        libd->plugin_argc++;
       }
       in_argument = 0;
     }
     else if (plugin_options[ichar] == ' ') {
       if ( ! in_double_quotes && ! in_single_quotes ) {
-	if ( in_argument ) {
-	  plugin_argc++;
-	}
-	in_argument = 0;
-	plugin_options[ichar] = '\0';
+        if ( in_argument ) {
+          libd->plugin_argc++;
+        }
+        in_argument = 0;
+        plugin_options[ichar] = '\0';
       }
     }
     else if (plugin_options[ichar] == '\"') {
       if ( in_single_quotes ) {
-	mdi_error("Nested quotes not supported by MDI_Launch_plugin \"options\" argument.");
+        mdi_error("Nested quotes not supported by MDI_Launch_plugin \"options\" argument.");
       }
       in_argument = 1;
       in_double_quotes = (in_double_quotes + 1) % 2;
@@ -235,7 +334,7 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
     }
     else if (plugin_options[ichar] == '\'') { 
       if ( in_double_quotes ) {
-	mdi_error("Nested quotes not supported by MDI_Launch_plugin \"options\" argument.");
+        mdi_error("Nested quotes not supported by MDI_Launch_plugin \"options\" argument.");
       }
       in_argument = 1;
       in_single_quotes = (in_single_quotes + 1) % 2;
@@ -247,29 +346,39 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   }
 
   // construct pointers to all of the arguments
-  plugin_argv = malloc( plugin_argc * sizeof(char*) );
+  libd->plugin_argv = malloc( libd->plugin_argc * sizeof(char*) );
+  libd->plugin_argv_allocated = 1;
   int iarg = 0;
   for (ichar=0; ichar < options_len; ichar++) {
     if ( plugin_options[ichar] != '\0' ) {
       if ( ichar == 0 || plugin_options[ichar-1] == '\0' ) {
-	plugin_argv[iarg] = &plugin_options[ichar];
-	iarg++;
+        libd->plugin_argv[iarg] = &plugin_options[ichar];
+        iarg++;
       }
     }
   }
-  if ( iarg != plugin_argc ) {
+  if ( iarg != libd->plugin_argc ) {
     mdi_error("Programming error: unable to correctly parse the MDI_Launch_plugin \"options\" argument.");
   }
 
-  //
-  // Get the path to the plugin
-  // Note: Eventually, should probably replace this code with libltdl
-  //
-  char* plugin_path = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+  return 0;
+}
+
+
+
+/*! \brief Launch an MDI plugin
+ *
+ */
+int library_launch_plugin(const char* plugin_name, const char* options, void* mpi_comm_ptr,
+                          MDI_Driver_node_callback_t driver_node_callback,
+                          void* driver_callback_object) {
+  int ret;
+  int driver_code_id = current_code;
+  code* this_code = get_code(driver_code_id);
+  MPI_Comm mpi_comm = *(MPI_Comm*) mpi_comm_ptr;
+
+
 
-  // Get the name of the plugin's init function
-  char* plugin_init_name = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
-  snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_init_%s", plugin_name);
 
   // initialize a communicator for the driver
   int icomm = library_initialize();
@@ -291,87 +400,40 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   // Set the mpi communicator associated with this plugin instance
   libd->mpi_comm = mpi_comm;
 
+  // Parse plugin command-line options
+  library_parse_options(options, libd);
+
+  // Assign the global command-line options variables to the values for this plugin  
+  plugin_argc = libd->plugin_argc;
+  plugin_argv = libd->plugin_argv;
+
+  //
+  // Get the path to the plugin
+  // Note: Eventually, should probably replace this code with libltdl
+  //
+  char* plugin_path = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+
+  // Get the name of the plugin's init function
+  char* plugin_init_name = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+  snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_init_%s", plugin_name);
+
+
+
 
   /*************************************************/
   /*************** BEGIN PLUGIN MODE ***************/
   /*************************************************/
   plugin_mode = 1;
 
-  // Attempt to load a python script
-  snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/%s.py", this_code->plugin_path, plugin_name);
-  if ( file_exists(plugin_path) ) {
-    ret = python_plugin_init( plugin_name, plugin_path, options, mpi_comm_ptr );
-    if ( ret != 0 ) {
-      mdi_error("Error in python_plugin_init");
-      return -1;
-    }
-  }
-  else {
-
-#ifdef _WIN32
-  // Attempt to open a library with a .dll extension
-  snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/lib%s.dll", this_code->plugin_path, plugin_name);
-  HINSTANCE plugin_handle = LoadLibrary( plugin_path );
-  if ( ! plugin_handle ) {
-    // Unable to find the plugin library
-    free( plugin_path );
-    mdi_error("Unable to open MDI plugin");
-    return -1;
-  }
-
-  // Load a plugin's initialization function
-  MDI_Plugin_init_t plugin_init = (MDI_Plugin_init_t) (intptr_t) GetProcAddress( plugin_handle, plugin_init_name );
-  if ( ! plugin_init ) {
-    free( plugin_path );
-    free( plugin_init_name );
-    FreeLibrary( plugin_handle );
-    mdi_error("Unable to load MDI plugin init function");
-    return -1;
-  }
-
-#else
-  // Attempt to open a library with a .so extension
-  snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/lib%s.so", this_code->plugin_path, plugin_name);
-  void* plugin_handle = dlopen(plugin_path, RTLD_NOW);
-  if ( ! plugin_handle ) {
-
-    // Attempt to open a library with a .dylib extension
-    snprintf(plugin_path, PLUGIN_PATH_LENGTH, "%s/lib%s.dylib", this_code->plugin_path, plugin_name);
-    plugin_handle = dlopen(plugin_path, RTLD_NOW);
-    if ( ! plugin_handle ) {
-      // Unable to find the plugin library
-      free( plugin_path );
-      free( plugin_init_name );
-      mdi_error("Unable to open MDI plugin");
-      return -1;
-    }
-  }
-
-  // Load a plugin's initialization function
-  MDI_Plugin_init_t plugin_init = (MDI_Plugin_init_t) (intptr_t) dlsym(plugin_handle, plugin_init_name);
-  if ( ! plugin_init ) {
-    free( plugin_path );
-    free( plugin_init_name );
-    dlclose( plugin_handle );
-    mdi_error("Unable to load MDI plugin init function");
-    return -1;
-  }
-#endif
-
-  // Initialize an instance of the plugin
-  ret = plugin_init();
-  if ( ret != 0 ) {
-    mdi_error("MDI plugin init function returned non-zero exit code");
-    return -1;
-  }
+  library_load_init(plugin_name, mpi_comm_ptr, libd, 0);
 
+  if (libd->is_python == 0 ) {
   // Close the plugin library
 #ifdef _WIN32
-  FreeLibrary( plugin_handle );
+    FreeLibrary( libd->plugin_handle );
 #else
-  dlclose( plugin_handle );
+    dlclose( libd->plugin_handle );
 #endif
-
   }
 
   /*************************************************/
@@ -388,11 +450,97 @@ int library_launch_plugin(const char* plugin_name, const char* options, void* mp
   free( plugin_path );
   free( plugin_init_name );
 
-  // free memory from storing the plugin's command-line options
-  free( plugin_options );
-  free( plugin_unedited_options );
-  free( plugin_argv );
+  return 0;
+}
 
+
+
+/*! \brief Open an MDI plugin in the background
+ *
+ */
+int library_open_plugin(const char* plugin_name, const char* options, void* mpi_comm_ptr,
+                          MDI_Comm* mdi_comm_ptr) {
+  int ret;
+  int driver_code_id = current_code;
+  code* this_code = get_code(driver_code_id);
+  MPI_Comm mpi_comm = *(MPI_Comm*) mpi_comm_ptr;
+
+  // initialize a communicator for the driver
+  int icomm = library_initialize();
+  communicator* driver_comm = get_communicator(current_code, icomm);
+  library_data* libd = (library_data*) driver_comm->method_data;
+  libd->connected_code = (int)codes.size;
+
+  MDI_Comm comm;
+  ret = MDI_Accept_Communicator(&comm);
+  if ( ret != 0 || comm == MDI_COMM_NULL ) {
+    mdi_error("MDI unable to create communicator for plugin");
+    return -1;
+  }
+
+  // Set the mpi communicator associated with this plugin instance
+  libd->mpi_comm = mpi_comm;
+
+  // Parse plugin command-line options
+  library_parse_options(options, libd);
+
+  // Assign the global command-line options variables to the values for this plugin  
+  plugin_argc = libd->plugin_argc;
+  plugin_argv = libd->plugin_argv;
+
+  //
+  // Get the path to the plugin
+  // Note: Eventually, should probably replace this code with libltdl
+  //
+  char* plugin_path = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+
+  // Get the name of the plugin's init function
+  char* plugin_init_name = malloc( PLUGIN_PATH_LENGTH * sizeof(char) );
+  snprintf(plugin_init_name, PLUGIN_PATH_LENGTH, "MDI_Plugin_open_%s", plugin_name);
+
+  /*************************************************/
+  /*************** BEGIN PLUGIN MODE ***************/
+  /*************************************************/
+  plugin_mode = 1;
+
+  library_load_init(plugin_name, mpi_comm_ptr, libd, 1);
+
+  /*************************************************/
+  /**************** END PLUGIN MODE ****************/
+  /*************************************************/
+  plugin_mode = 0;
+  current_code = driver_code_id;
+
+  // Delete the driver's communicator to the engine
+  // This will also delete the engine code and its communicator
+  //delete_communicator(driver_code_id, comm);
+
+  // free memory from loading the plugin's initialization function
+  free( plugin_path );
+  free( plugin_init_name );
+
+  *mdi_comm_ptr = comm;
+  return 0;
+}
+
+int library_close_plugin(MDI_Comm mdi_comm) {
+  code* this_code = get_code(current_code);
+  communicator* this_comm = get_communicator(this_code->id, mdi_comm);
+  library_data* libd = (library_data*) this_comm->method_data;
+
+  if (libd->is_python == 0 ) {
+  // Close the plugin library
+#ifdef _WIN32
+    FreeLibrary( libd->plugin_handle );
+#else
+    dlclose( libd->plugin_handle );
+#endif
+  }
+
+  // Delete the driver's communicator to the engine
+  // This will also delete the engine code and its communicator
+  delete_communicator(current_code, mdi_comm);
+  
   return 0;
 }
 
@@ -413,6 +561,8 @@ int library_initialize() {
   new_comm->mdi_version[0] = MDI_MAJOR_VERSION;
   new_comm->mdi_version[1] = MDI_MINOR_VERSION;
   new_comm->mdi_version[2] = MDI_PATCH_VERSION;
+  new_comm->name_length = MDI_NAME_LENGTH;
+  new_comm->command_length = MDI_COMMAND_LENGTH;
 
   // allocate the method data
   library_data* libd = malloc(sizeof(library_data));
@@ -420,6 +570,7 @@ int library_initialize() {
   libd->buf_allocated = 0;
   libd->execute_on_send = 0;
   libd->mpi_comm = MPI_COMM_NULL;
+  libd->plugin_argv_allocated = 0;
   new_comm->method_data = libd;
 
   // if this is an engine, go ahead and set the driver as the connected code
@@ -432,26 +583,28 @@ int library_initialize() {
 
     // set the engine's mpi communicator
     if ( plugin_mode ) {
+      // get the driver's library data
       code* driver_code = get_code(driver_code_id);
       MDI_Comm matching_handle = library_get_matching_handle(comm_id);
       communicator* driver_comm = get_communicator(driver_code->id, matching_handle);
       library_data* driver_libd = (library_data*) driver_comm->method_data;
+
       libd->mpi_comm = driver_libd->mpi_comm;
       this_code->intra_MPI_comm = libd->mpi_comm;
 
-      // Check whether MPI has been initialized
+      // check whether MPI has been initialized
       int mpi_init_flag;
       if ( MPI_Initialized(&mpi_init_flag) ) {
-	mdi_error("Error in MDI_Init: MPI_Initialized failed");
-	return 1;
+        mdi_error("Error in MDI_Init: MPI_Initialized failed");
+        return 1;
       }
 
       // Set the engine's MPI rank
       if ( mpi_init_flag == 1 ) {
-	MPI_Comm_rank( this_code->intra_MPI_comm, &this_code->intra_rank );
+        MPI_Comm_rank( this_code->intra_MPI_comm, &this_code->intra_rank );
       }
       else {
-	this_code->intra_rank = 0;
+        this_code->intra_rank = 0;
       }
     }
   }
@@ -603,7 +756,7 @@ int library_set_command(const char* command, MDI_Comm comm) {
 
   // set the command
   library_data* engine_lib = (library_data*) engine_comm->method_data;
-  snprintf(engine_lib->command, COMMAND_LENGTH, "%s", command);
+  snprintf(engine_lib->command, MDI_COMMAND_LENGTH_, "%s", command);
 
   return 0;
 }
@@ -878,6 +1031,10 @@ int communicator_delete_lib(void* comm) {
   if ( this_code->is_library == 0 ) {
     delete_code(libd->connected_code);
   }
+  
+  if ( libd->plugin_argv_allocated ) {
+    free( libd->plugin_argv );
+  }
 
   // delete the method-specific information
   if ( libd->buf_allocated ) {
